unit IpxMisc;

{ This unit contains miscellaneous IPX functions }

interface

uses IpxEsr;

const IPX_PACKET          : Byte = 4;
      IPX_BROADCAST_PACKET: Byte = 20;
      SPX_PACKET          : Byte = 5;

function alloc_ECB(PacketType: Byte; SocketNumber: Word;
                   BufferSize: Word; Msg : Word): PXECB;
procedure free_ECB(ECB: PXECB);

implementation

uses WinProcs, WinTypes, NXTW;

{ This function allocates a fixed memory block for an }
{ ECB and a data buffer if one is requested           }
function alloc_ECB(PacketType: Byte; SocketNumber: Word; BufferSize: Word;
                   msg : Word): PXECB;
var PECBData: PXECB;
    PIpxHead: PIPXHeader;
    PBuffer : pointer;
begin
   PECBData := GlobalAllocPtr(GMEM_FIXED or GMEM_ZEROINIT or GMEM_DDESHARE, SizeOf(XECB));
   GlobalPageLock(SelectorOf(PECBData));
   Result := PECBData;

   if PECBData = nil then
      Exit;

   if (PacketType = IPX_PACKET) or (PacketType = IPX_BROADCAST_PACKET) then begin
      PIPXHead:= GlobalAllocPtr(GMEM_FIXED or GMEM_ZEROINIT or GMEM_DDESHARE, SizeOf(IPXHeader));
      if PIPXHead = nil then begin
         free_ECB(PECBData);
         Result := nil;
         Exit;
         end;
      PECBData^.BaseECB.FragmentDescriptor[0].size:= SizeOf(IPXHeader);
      end
   else begin
      PIPXHead:= GlobalAllocPtr(GMEM_FIXED or GMEM_ZEROINIT or GMEM_DDESHARE, SizeOf(SPXHeader));
      if PIPXHead = nil then begin
         free_ECB(PECBData);
         Result := nil;
         Exit;
         end;
      PECBData^.BaseECB.FragmentDescriptor[0].size:= SizeOf(SPXHeader);
      end;

   GlobalPageLock(SelectorOf(PIPXHead));
   PIPXHead^.PacketType := PacketType;
   PECBData^.BaseECB.FragmentCount := 1;
   PECBData^.BaseECB.FragmentDescriptor[0].address := PIPXHead;
   if BufferSize <> 0 then begin
       PBuffer:= GlobalAllocPtr(GMEM_FIXED or GMEM_ZEROINIT or GMEM_DDESHARE, BufferSize);
       if PBuffer = nil then begin
          free_ECB(PECBData);
          Result := nil;
          Exit;
          end;
       GlobalPageLock(SelectorOf(PBuffer));
       with PECBData^.BaseECB do begin
          FragmentCount := 2;
          FragmentDescriptor[1].address := PBuffer;
          FragmentDescriptor[1].size    := BufferSize;
          end;
       end;
   PECBData^.BaseECB.EsrAddress   := Esr;
   PECBData^.BaseECB.SocketNumber := SocketNumber;
   PECBData^.Msg := Msg;
end;

{ This function unlocks and frees an ECB }
procedure free_ECB(ECB: PXECB);
begin
   with ECB^.BaseECB do begin
     if FragmentDescriptor[0].size <> 0 then
        GlobalPageUnlock(SelectorOf(FragmentDescriptor[0].address));
     if FragmentDescriptor[1].size <> 0 then
        GlobalPageUnlock(SelectorOf(FragmentDescriptor[1].address));
     GlobalPageUnlock(SelectorOf(ECB));
     if FragmentDescriptor[0].size <> 0 then
          GlobalFreePtr(FragmentDescriptor[0].address);
     if FragmentDescriptor[1].size <> 0 then
          GlobalFreePtr(FragmentDescriptor[1].address);
     end;
   GlobalFreePtr(ECB);
end;

end.

