unit Ipxsap1;

{ Novell's Service Advertising Protocol requires that all }
{ server's i.e. file servers, application servers, etc.,  }
{ advertise their presence by sending out an IPX packet   }
{ containing certain information at least once every 60   }
{ seconds.  Also, servers are required to respond on      }
{ on demand at the receipt of a special query packet.     }
{ This example listens for and displays SAP information   }
{ and will transmit a query packet if desired.            }
{ Incidentally, this protocol is used by the shell to     }
{ locate a file server to attach to after your network    }
{ drivers are first loaded.  Server type 4 is a NetWare   }
{ file server.                                            }

interface

uses SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics,
     Controls, Forms, Dialogs, NXTW, StdCtrls, NWCalDef, IpxEsr,
     IpxMisc, ExtCtrls, Buttons, Grids;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    SpeedButton1: TSpeedButton;
    StringGrid1: TStringGrid;
    SpeedButton2: TSpeedButton;
    Header1: THeader;
    procedure FormShow(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure ECBComplete(var TheMessage : TMessage); message WM_IPXECBCOMPLETE;
  end;

var
  Form1    : TForm1;
  IPXTaskID: DWord;
  MyECBs   : array[0..4] of PXECB;
  SocketNum: Word;

implementation

{$R *.DFM}

procedure TForm1.ECBComplete(var TheMessage : TMessage);
var MyECB: PXECB;
    Head : PIPXHeader;
    SAP  : PSapServiceID;
    SName: array[0..49] of char;
    SType: Word;
    Tmp  : string;
    ServersInPacket: Integer;
begin
   MyECB := PXECB(TheMessage.LParam);
   with MyECB^.BaseECB do begin
      Head := PIPXHeader(FragmentDescriptor[0].address);
      SAP  := PSapServiceID(FragmentDescriptor[1].address);
      end;
   { Make sure that this is a response and not a query }
   if (SAP^.ResponseType = $0200) or (SAP^.ResponseType = $0400) then begin
      { A packet contain up to seven server names }
      ServersInPacket := (swap(Head^.Length)-32) div SizeOf(TSapServer);
      dec(ServersInPacket);
      while ServersInPacket >= 0 do begin
         with SAP^.Server[ServersInPacket] do begin
            StrLCopy(SName, ServerName, SizeOf(ServerName));
            SType := ServerType;
            StringGrid1.RowCount := StringGrid1.RowCount + 1;
            StringGrid1.Cells[0,StringGrid1.RowCount-2] := StrPas(SName);
            StringGrid1.Cells[1,StringGrid1.RowCount-2] := IntToHex(Swap(SType), 4);
            Tmp := IntToHex(Ord(Network[0]),2) + IntToHex(Ord(Network[1]),2) +
                   IntToHex(Ord(Network[2]),2) + IntToHex(Ord(Network[3]),2);
            StringGrid1.Cells[2,StringGrid1.RowCount-2] := Tmp;
            Tmp := IntToHex(Ord(Node[0]),2) + IntToHex(Ord(Node[1]),2) +
                   IntToHex(Ord(Node[2]),2) + IntToHex(Ord(Node[3]),2) +
                   IntToHex(Ord(Node[4]),2) + IntToHex(Ord(Node[5]),2);
            StringGrid1.Cells[3,StringGrid1.RowCount-2] := Tmp;
            StringGrid1.Cells[4,StringGrid1.RowCount-2] := IntToHex(swap(Socket), 4);
            ServersInPacket := ServersInPacket - 1;
            end;
         end;
      end;
   IPXListenForPacket(IPXTaskID, PECB(MyECB)^);
end;

procedure TForm1.FormShow(Sender: TObject);
var Count: Byte;
begin
   if IPXInitialize(IPXTaskID, 0, 0) <> 0 then begin
      MessageDlg('IPX DRIVERS WERE NOT FOUND', mtError, [mbOK], 0);
      Close;
      end;
   { $452 = SAP SOCKET }
   SocketNum := swap($0452);
   for Count := 0 to 4 do begin
      MyECBs[Count] := alloc_ECB(IPX_PACKET, SocketNum, 576, WM_IPXECBCOMPLETE);
      MyECBs[Count]^.Wnd := Handle;
      end;
   IPXOpenSocket(IPXTaskID, SocketNum, 0);
   for Count := 0 to 3 do
      IPXListenForPacket(IPXTaskID, PECB(MyECBs[Count])^);
end;

procedure TForm1.FormDestroy(Sender: TObject);
var Count: Byte;
begin
    for Count := 0 to 4 do
       IPXCancelEvent(IPXTaskID, PECB(MyECBs[Count])^);
    IPXCloseSocket(IPXTaskID,SocketNum);
    for Count := 0 to 4 do
       free_ECB(MyECBs[Count]);
    IPXSPXDeinit(IPXTaskID);
end;

procedure TForm1.SpeedButton1Click(Sender: TObject);
begin
   StringGrid1.RowCount   := 1;
   StringGrid1.Cells[0,0] := '';
   StringGrid1.Cells[1,0] := '';
   StringGrid1.Cells[2,0] := '';
   StringGrid1.Cells[3,0] := '';
   StringGrid1.Cells[4,0] := '';
end;

procedure TForm1.SpeedButton2Click(Sender: TObject);
var SapQuery: PSAPQuery;
    Head    : PIPXHeader;
    Time    : Integer;
    ECB     : PECB;
begin
   { Clear out the grid }
   SpeedButton1Click(Sender);

   ECB := @MyECBs[4]^.BaseECB;
   SapQuery := PSapQuery(ECB^.FragmentDescriptor[1].address);
   SapQuery^.QueryType := $0100;   { General Query    }
   SapQuery^.ServerType := $FFFF;  { All Server Types }
   ECB^.FragmentDescriptor[1].size := SizeOf(TSAPQuery);

   Head := PIPXHeader(ECB^.FragmentDescriptor[0].address);
   { Fill the destination address with our local address }
   IPXGetInternetworkAddress(IPXTaskID, Head^.Destination);
   { Now change destination node to $FF = Broadcast to all }
   FillChar(Head^.Destination.Node, SizeOf(Head^.Destination.Node), $FF);
   { Now change destination socket to $0452 = SAP SOCKET }
   Head^.Destination.Socket[0] := $04;
   Head^.Destination.Socket[1] := $52;

   { Get the local router's address (usually your file server) }
   IPXGetLocalTarget(IPXTaskID, Head^.Destination,
                     PIPXAddress(@ECB^.ImmediateAddress)^, Time);

   { We do not want our ESR executed }
   ECB^.EsrAddress := nil;

   IPXSendPacket(IPXTaskID, PECB(MyECBs[4])^);
end;

end.

